﻿#----------------------------------------------------------#
#- MusicBeeIPCSDK Rb v2.0.0                               -#
#- Copyright © Kerli Low 2014                             -#
#- This file is licensed under the                        -#
#- BSD 2-Clause License                                   -#
#- See LICENSE_MusicBeeIPCSDK for more information.       -#
#----------------------------------------------------------#

require_relative "constants.rb"
require_relative "structs.rb"


class MusicBeeIPC
  private
  
  # ----------------------------------------------------------------------------
  # All strings are encoded in UTF-16 little endian
  # ----------------------------------------------------------------------------

  # -Int32: 32 bit integer
  # -Int32: 32 bit integer
  # -...
  def pack_i(*int32s)
    cds = COPYDATASTRUCT.new
    cds[:dwData] = 0
    
    num = int32s.length
    
    cds[:cbData] = MBIPC_SIZEOFINT * num
    
    data = int32s.pack("l" + num.to_s)
    
    cds[:lpData] = FFI::MemoryPointer.from_string(data)
    
    return cds.to_ptr.address
  end

  # -Int32:  Byte count of string
  # -byte[]: String data
  # -Int32:  Byte count of string
  # -byte[]: String data
  # -...
  def pack_s(*strings)
    cds = COPYDATASTRUCT.new
    cds[:dwData] = 0
    
    num = strings.length
    
    cds[:cbData] = MBIPC_SIZEOFINT * num
    
    encoded = strings.map do |s|
      e = s.encode("UTF-16LE").force_encoding("BINARY")
      cds[:cbData] += e.length
      e
    end
    
    data = ""
    encoded.each { |e| data += [e.length].pack("l") + e }
    
    cds[:lpData] = FFI::MemoryPointer.from_string(data)
    
    return cds.to_ptr.address
  end

  # -Int32:  Byte count of string
  # -byte[]: String data
  # -Int32:  32 bit integer
  # -Int32:  32 bit integer
  # -...
  def pack_si(string_1, *int32s)
    cds = COPYDATASTRUCT.new
    cds[:dwData] = 0
    
    encoded = string_1.encode("UTF-16LE").force_encoding("BINARY")
    byte_count = encoded.length
    
    num = int32s.length
    
    cds[:cbData] = MBIPC_SIZEOFINT * (num + 1) + byte_count
    
    data = [byte_count].pack("l") + encoded +
           int32s.pack("l" + int32s.length.to_s)
    
    cds[:lpData] = FFI::MemoryPointer.from_string(data)
    
    return cds.to_ptr.address
  end

  # -Int32:  Byte count of string
  # -byte[]: String data
  # -Int32:  bool
  # -Int32:  bool
  # -...
  def pack_sb(string_1, *bools)
    cds = COPYDATASTRUCT.new
    cds[:dwData] = 0
    
    encoded = string_1.encode("UTF-16LE").force_encoding("BINARY")
    byte_count = encoded.length
    
    bools_i = bools.map { |b| b ? 1 : 0 }
    
    num = bools_i.length
    
    cds[:cbData] = MBIPC_SIZEOFINT * (num + 1) + byte_count
    
    data = [byte_count].pack("l") + encoded +
           bools_i.pack("l" + bools_i.length.to_s)
    
    cds[:lpData] = FFI::MemoryPointer.from_string(data)
    
    return cds.to_ptr.address
  end

  # -Int32:  Byte count of string
  # -byte[]: String data
  # -double: 64-bit floating-point value
  # -double: 64-bit floating-point value
  # -...
  def pack_sd(string_1, *doubles)
    cds = COPYDATASTRUCT.new
    cds[:dwData] = 0
    
    encoded = string_1.encode("UTF-16LE").force_encoding("BINARY")
    byte_count = encoded.length
    
    num = doubles.length
    
    cds[:cbData] = MBIPC_SIZEOFDOUBLE * num + MBIPC_SIZEOFINT + byte_count
    
    data = [byte_count].pack("l") + encoded +
           doubles.pack("d" + doubles.length.to_s)
    
    cds[:lpData] = FFI::MemoryPointer.from_string(data)
    
    return cds.to_ptr.address
  end

  # -Int32:  Byte count of string
  # -byte[]: String data
  # -Int32:  Number of strings in string array
  # -Int32:  Byte count of string in string array
  # -byte[]: String data in string array
  # -Int32:  Byte count of string in string array
  # -byte[]: String data in string array
  # -...
  def pack_ssa(string_1, strings)
    cds = COPYDATASTRUCT.new
    cds[:dwData] = 0
    
    encoded_1 = string_1.encode("UTF-16LE").force_encoding("BINARY")
    byte_count_1 = encoded_1.length
    
    num = strings.length
    
    cds[:cbData] = MBIPC_SIZEOFINT * (num + 2) + byte_count_1
    
    encoded = strings.map do |s|
      e = s.encode("UTF-16LE").force_encoding("BINARY")
      cds[:cbData] += e.length
      e
    end
    
    data = [byte_count_1].pack("l") + encoded_1 + [num].pack("l")
    encoded.each { |e| data += [e.length].pack("l") + e }
    
    cds[:lpData] = FFI::MemoryPointer.from_string(data)
    
    return cds.to_ptr.address
  end

  # -Int32:  Byte count of string
  # -byte[]: String data
  # -Int32:  Byte count of string
  # -byte[]: String data
  # -Int32:  Number of strings in string array
  # -Int32:  Byte count of string in string array
  # -byte[]: String data in string array
  # -Int32:  Byte count of string in string array
  # -byte[]: String data in string array
  # -...
  def pack_sssa(string_1, string_2, strings)
    cds = COPYDATASTRUCT.new
    cds[:dwData] = 0
    
    encoded_1 = string_1.encode("UTF-16LE").force_encoding("BINARY")
    byte_count_1 = encoded_1.length
    
    encoded_2 = string_2.encode("UTF-16LE").force_encoding("BINARY")
    byte_count_2 = encoded_2.length
    
    num = strings.length
    
    cds[:cbData] = MBIPC_SIZEOFINT * (num + 3) + byte_count_1 + byte_count_2
    
    encoded = strings.map do |s|
      e = s.encode("UTF-16LE").force_encoding("BINARY")
      cds[:cbData] += e.length
      e
    end
    
    data = [byte_count_1].pack("l") + encoded_1 +
           [byte_count_2].pack("l") + encoded_2 + [num].pack("l")
    encoded.each { |e| data += [e.length].pack("l") + e }
    
    cds[:lpData] = FFI::MemoryPointer.from_string(data)
    
    return cds.to_ptr.address
  end

  # -Int32:  Byte count of string
  # -byte[]: String data
  # -Int32: 32 bit integer
  # -Int32:  Byte count of string
  # -byte[]: String data
  # -...
  def pack_sis(string_1, int32_1, string_2)
    cds = COPYDATASTRUCT.new
    cds[:dwData] = 0
    
    encoded_1 = string_1.encode("UTF-16LE").force_encoding("BINARY")
    byte_count_1 = encoded_1.length
    
    encoded_2 = string_2.encode("UTF-16LE").force_encoding("BINARY")
    byte_count_2 = encoded_2.length
    
    cds[:cbData] = MBIPC_SIZEOFINT * 3 + byte_count_1 + byte_count_2
    
    data = [byte_count_1].pack("l") + encoded_1 +
           [int32_1, byte_count_2].pack("ll") + encoded_2
    
    cds[:lpData] = FFI::MemoryPointer.from_string(data)
    
    return cds.to_ptr.address
  end

  # -Int32: Number of integers in integer array
  # -Int32: 32 bit integer
  # -Int32: 32 bit integer
  # -...
  # -Int32: 32 bit integer
  def pack_iai(int32s, int32_1)
    cds = COPYDATASTRUCT.new
    cds[:dwData] = 0
    
    num = int32s.length
    
    cds[:cbData] = MBIPC_SIZEOFINT * (num + 2)
    
    data = ([num] + int32s + [int32_1]).pack("l" + (num + 2).to_s)
    
    cds[:lpData] = FFI::MemoryPointer.from_string(data)
    
    return cds.to_ptr.address
  end

  # -Int32:  Byte count of string
  # -byte[]: String data
  # -Int32:  Number of integers in integer array
  # -Int32:  32 bit integer
  # -Int32:  32 bit integer
  # -...
  # -Int32:  32 bit integer
  def pack_siai(string_1, int32s, int32_1)
    cds = COPYDATASTRUCT.new
    cds[:dwData] = 0
    
    encoded = string_1.encode("UTF-16LE").force_encoding("BINARY")
    byte_count = encoded.length
    
    num = int32s.length
    
    cds[:cbData] = MBIPC_SIZEOFINT * (num + 3) + byte_count
    
    data = [byte_count].pack("l") + encoded +
           ([num] + int32s + [int32_1]).pack("l" + (num + 2).to_s)
    
    cds[:lpData] = FFI::MemoryPointer.from_string(data)
    
    return cds.to_ptr.address
  end
end
