﻿#----------------------------------------------------------#
#- MusicBeeIPCSDK Rb v2.0.0                               -#
#- Copyright © Kerli Low 2014                             -#
#- This file is licensed under the                        -#
#- BSD 2-Clause License                                   -#
#- See LICENSE_MusicBeeIPCSDK for more information.       -#
#----------------------------------------------------------#

require_relative "constants.rb"
require_relative "structs.rb"


class MusicBeeIPC
  private
  
  # ----------------------------------------------------------------------------
  # All strings are encoded in UTF-16 little endian
  # ----------------------------------------------------------------------------

  # -Int32:  Byte count of the string
  # -byte[]: String data
  # Free lr after use
  #
  # @return [String]
  def unpack_s(lr)
    string_1 = ""
    
    begin
      mmf = open_mmf(lr)
      raise "Failed to open MMF." if !mmf
      
      view, ptr = map_mmf_view(mmf, lr)
      raise "Failed to map MMF view." if !view
    rescue
      return ""
    ensure
      close_mmf(mmf) if mmf
    end
    
    begin
      byte_count = ptr.read_string(MBIPC_SIZEOFINT).unpack("l")[0]
      ptr += MBIPC_SIZEOFINT
      
      if byte_count > 0
        string_1 = ptr.read_string(byte_count).force_encoding("UTF-16LE").
                   encode(__ENCODING__)
      end
    rescue
      return ""
    ensure
      unmap_mmf_view(view)
      close_mmf(mmf)
    end
    
    return string_1
  end

  # -Int32:  Number of strings
  # -Int32:  Byte count of 1st string
  # -byte[]: 1st string data
  # -Int32:  Byte count of 2nd string
  # -byte[]: 2nd string data
  # -...
  # Free lr after use
  #
  # @return [Array<String>]
  def unpack_sa(lr)
    begin
      mmf = open_mmf(lr)
      raise "Failed to open MMF." if !mmf
      
      view, ptr = map_mmf_view(mmf, lr)
      raise "Failed to map MMF view." if !view
    rescue
      return []
    ensure
      close_mmf(mmf) if mmf
    end
    
    begin
      str_count = ptr.read_string(MBIPC_SIZEOFINT).unpack("l")[0]
      ptr += MBIPC_SIZEOFINT
      
      strings = Array.new(str_count)
      
      strings.map! do |s|
        byte_count = ptr.read_string(MBIPC_SIZEOFINT).unpack("l")[0]
        ptr += MBIPC_SIZEOFINT
      
        if byte_count > 0
          s = ptr.read_string(byte_count).force_encoding("UTF-16LE").
              encode(__ENCODING__)
          ptr += byte_count
          s
        end
      end
    rescue
      return []
    ensure
      unmap_mmf_view(view)
      close_mmf(mmf)
    end
    
    return strings
  end

  # -Int32: 1st 32 bit integer
  # -Int32: 2nd 32 bit integer
  # Free lr after use
  #
  # @return [Array<int>]
  def unpack_ii(lr)
    begin
      mmf = open_mmf(lr)
      raise "Failed to open MMF." if !mmf
      
      view, ptr = map_mmf_view(mmf, lr)
      raise "Failed to map MMF view." if !view
    rescue
      return [-1, -1]
    ensure
      close_mmf(mmf) if mmf
    end
    
    begin
      ints = ptr.read_string(MBIPC_SIZEOFINT * 2).unpack("ll")
    rescue
      return [-1, -1]
    ensure
      unmap_mmf_view(view)
      close_mmf(mmf)
    end
    
    return ints
  end

  # -Int32: Number of integers
  # -Int32: 1st 32 bit integer
  # -Int32: 2nd 32 bit integer
  # -...
  # Free lr after use
  #
  # @return [Array<int>]
  def unpack_ia(lr)
    begin
      mmf = open_mmf(lr)
      raise "Failed to open MMF." if !mmf
      
      view, ptr = map_mmf_view(mmf, lr)
      raise "Failed to map MMF view." if !view
    rescue
      return []
    ensure
      close_mmf(mmf) if mmf
    end
    
    begin
      int_count = ptr.read_string(MBIPC_SIZEOFINT).unpack("l")[0]
      ptr += MBIPC_SIZEOFINT
      
      ints = ptr.read_string(MBIPC_SIZEOFINT * int_count).
                 unpack("l" + int_count.to_s)
    rescue
      return []
    ensure
      unmap_mmf_view(view)
      close_mmf(mmf)
    end
    
    return ints
  end
  
  
  def open_mmf(lr)
    return nil if !lr || lr == 0
    
    ls = LRUShort.new(lr)
    
    name = ("mbipc_mmf_" + ls[:low].to_s + "\0").encode("UTF-16LE")

    # FILE_MAP_READ = 0x0004 = 4
    # FALSE = 0
    open_file_mapping(4, 0, name)
  end
  
  def close_mmf(mmf)
    close_handle(mmf)
  end
  
  def map_mmf_view(mmf, lr)
    ls = LRUShort.new(lr)
    
    # FILE_MAP_READ = 0x0004 = 4
    view = map_view_of_file(mmf, 4, 0, 0, 0)
    
    ptr = FFI::Pointer.new(:char, view + ls[:high] + MBIPC_SIZEOFLONG)
    
    return [view, ptr]
  end
  
  def unmap_mmf_view(view)
    unmap_view_of_file(view)
  end
end
